@@ -23,6 +23,8 @@
 image *undo_ring[MAX_UNDO];
 /* undo_n and undo_cur are relative to undo_start, mod MAX_UNDO. */
 int undo_start, undo_n, undo_cur;
+/* Is there an undo in progress ? */
+int undo_current = 0;
 
 /* Stuff for the hi-res bitmap being drawn upon */
 /* It's stored in a runlength encoded form. Each int32 represents WHITE_X
@@ -580,6 +582,9 @@ void bitmap_clear ()
      mutating that rectangle. Copies needed parts of that rectangle
      to the current undo buffer.
 
+   undo_end ()
+     Finishes the undo transaction.
+
    undo_reset ()
      Clears the undo buffer.
 
@@ -628,6 +633,8 @@ void undo_begin (void) {
   /* Don't need to update any lines, because the region is empty. */
   undo_n++;
   undo_cur = undo_n;
+
+  undo_current = 1;
 }
 
 /* Make a new copy of the line, and copy it to the dst image. */
@@ -647,6 +654,8 @@ void undo_update (rect *rec) {
   int y;
   rect old_rec, clip_rec;
 
+  g_return_if_fail (undo_current);
+
   if (undo_n < 1) {
     fprintf (stderr, "undo_update: undo_n < 1\n");
     exit (1);
@@ -668,6 +677,12 @@ void undo_update (rect *rec) {
   rect_union (&(undo_ring[cur]->rec), &old_rec, &clip_rec);
 }
 
+void undo_end (void) {
+  g_return_if_fail (undo_current);
+
+  undo_current = 0;
+}
+
 void undo_reset (void) {
   while (undo_n) {
     undo_n--;
@@ -694,6 +709,9 @@ void undo () {
   if (undo_cur == 0) return;
   undo_cur--;
   undo_exchange ((undo_start + undo_cur) % MAX_UNDO);
+
+  if (undo_current)
+    undo_begin ();
 }
 
 void redo () {
@@ -701,4 +719,7 @@ void redo () {
   if (undo_cur == undo_n) return;
   undo_exchange ((undo_start + undo_cur) % MAX_UNDO);
   undo_cur++;
+
+  if (undo_current)
+    undo_begin ();
 }
