@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define ROUND(A) floor(A+0.5)
+#define ROUND(A) floor((A)+0.5)
 
 static Blob *
 blob_new (int y, int height)
@@ -306,6 +306,102 @@ blob_convex_union (Blob *b1, Blob *b2)
   return result;
 }
 
+static void
+blob_line_add_pixel (Blob *b, int x, int y)
+{
+  if (b->data[y-b->y].left > b->data[y-b->y].right)
+    b->data[y-b->y].left = b->data[y-b->y].right = x;
+  else
+    {
+      b->data[y-b->y].left = MIN (b->data[y-b->y].left, x);
+      b->data[y-b->y].right = MAX (b->data[y-b->y].right, x);
+    }
+}
+
+void
+blob_line (Blob *b, int x0, int y0, int x1, int y1)
+{
+  int dx, dy, d;
+  int incrE, incrNE;
+  int x, y;
+
+  int xstep = 1;
+  int ystep = 1;
+  
+  dx = x1 - x0;
+  dy = y1 - y0;
+  
+  if (dx < 0)
+    {
+      dx = -dx;
+      xstep = -1;
+    }
+
+  if (dy < 0)
+    {
+      dy = -dy;
+      ystep = -1;
+    }
+
+  /*  for (y = y0; y != y1 + ystep ; y += ystep)
+    {
+      b->data[y-b->y].left = 0;
+      b->data[y-b->y].right = -1;
+      }*/
+
+  x = x0;
+  y = y0;
+
+  if (dy < dx)
+    {
+      d = 2*dy - dx;		/* initial value of d */
+      incrE = 2 * dy;		/* increment used for move to E */
+      incrNE = 2 * (dy-dx);		/* increment used for move to NE */
+      
+      blob_line_add_pixel (b, x, y);
+      
+      while (x != x1)
+	{
+	  if (d <= 0)
+	    {
+	      d += incrE;
+	      x += xstep;
+	    }
+	  else
+	    {
+	      d += incrNE;
+	      x += xstep;
+	      y += ystep;
+	    }
+	  blob_line_add_pixel (b, x, y);
+	}
+    }
+  else
+    {
+      d = 2*dx - dy;		/* initial value of d */
+      incrE = 2 * dx;		/* increment used for move to E */
+      incrNE = 2 * (dx-dy);		/* increment used for move to NE */
+      
+      blob_line_add_pixel (b, x, y);
+      
+      while (y != y1)
+	{
+	  if (d <= 0)
+	    {
+	      d += incrE;
+	      y += ystep;
+	    }
+	  else
+	    {
+	      d += incrNE;
+	      x += xstep;
+	      y += ystep;
+	    }
+	  blob_line_add_pixel (b, x, y);
+	}
+    }
+}
+
 /****************************************************************
  * Code to scan convert an arbitrary ellipse into a Blob. Based
  * on Van Aken's conic algorithm in Foley and Van Damn 
@@ -373,9 +469,6 @@ blob_conic (Blob *b, int xs, int ys,
   int d,u,v,k1,k2,k3;		/* decision variables and increments */
   int octantCount;		/* number of octants to be drawn */
   int count;			/* number of steps for last octant */
-  int fx,dfx;			/* check x-boundaries */
-  int fy,dfy;			/* check y-boundaries */
-  int ddf;			/* second difference for boundary-checking */
   int tmp, i;
 
   EdgeType *present;
@@ -494,12 +587,6 @@ blob_conic (Blob *b, int xs, int ys,
      for the conic as quadratics in x and y, and when they go negative
      we know we are beyond the boundaries of the conic. */
 
-  ddf = 2*(B*B - 4*A*C);
-  fx = E*E - 4*C*F;
-  dfx = ddf/2 + dxdiag * (2*B*E - 4*C*D);
-  fy = D*D - 4*A*F;
-  dfy = ddf/2 + dydiag * (2*B*D - 4*A*E);
-
   while (1)
     {
       if (octantCount == 0)
@@ -541,16 +628,6 @@ blob_conic (Blob *b, int xs, int ys,
 		  u += k1;
 		  v += k2;
 		  d += u;
-		  if (dxsquare)
-		    {
-		      fx += dfx;
-		      dfx += ddf;
-		    }
-		  if (dysquare)
-		    {
-		      fy += dfy;
-		      dfy += ddf;
-		    }
 		}
 	      else
 		{
@@ -558,37 +635,6 @@ blob_conic (Blob *b, int xs, int ys,
 		  u += k2;
 		  v += k3;
 		  d += v;
-		  fx += dfx;
-		  dfx += ddf;
-		  fy += dfy;
-		  dfy += ddf;
-#ifdef NOHOP
-		  /* Check for octant hopping */
-		  if (u < v && fx >= 0 && fy >= 0)
-		    {
-		      /* hopped diag across ellipse, take square instead */
-		      x += dxsquare - dxdiag;
-		      y += dysquare - dydiag;
-		      d -= v;
-		      u += k1 - k2;
-		      v += k2 - k3;
-		      d += u;
-		      dfx -= ddf;
-		      fx -= dfx;
-		      dfy -= ddf;
-		      fy -= dfy;
-		      if (dxsquare)
-			{
-			  fx += dfx;
-			  dfx += ddf;
-			}
-		      if (dysquare)
-			{
-			  fy += dfy;
-			  dfy += ddf;
-			}
-		    }
-#endif
 		}
 	      if (count && --count == 0)
 		goto done;
@@ -614,10 +660,6 @@ blob_conic (Blob *b, int xs, int ys,
 		  u += k2;
 		  v += k3;
 		  d += v;
-		  fx += dfx;
-		  dfx += ddf;
-		  fy += dfy;
-		  dfy += ddf;
 		}
 	      else
 		{
@@ -625,47 +667,6 @@ blob_conic (Blob *b, int xs, int ys,
 		  u += k1;
 		  v += k2;
 		  d += u;
-		  if (dxsquare)
-		    {
-		      fx += dfx;
-		      dfx += ddf;
-		    }
-		  if (dysquare)
-		    {
-		      fy += dfy;
-		      dfy += ddf;
-		    }
-#ifdef NOHOP
-		  /* Check for octant hopping */
-		   if ((fx >= 0) && (fy >= 0) &&
-		       ((((octant == 2) || (octant == 6)) &&
-			 2*u > ROUND(v + B/2 + C)) ||
-			(((octant == 4) || (octant == 8)) &&
-			 2*u > ROUND(v - B/2 + A))))
-		       {
-			 /* hopped square across ellipse, take diag instead */
-			 x += dxdiag - dxsquare;
-			 y += dydiag - dysquare;
-			 d -= u;
-			 u += k2 - k1;
-			 v += k3 - k2;
-			 d += v;
-			 if (dxsquare)
-			   {
-			     dfx -= ddf;
-			     fx -= dfx;
-			   }
-			 if (dysquare)
-			   {
-			     dfy -= ddf;
-			     fy -= dfy;
-			   }
-			 fx += dfx;
-			 dfx += ddf;
-			 fy += dfy;
-			 dfy += ddf;
-		       }
-#endif
 		}
 	      if (count && --count <= 0)
 		goto done;
@@ -680,17 +681,12 @@ blob_conic (Blob *b, int xs, int ys,
 	  tmp = dxdiag;
 	  dxdiag = -dydiag;
 	  dydiag = tmp;
-	  /* Change appropriate boundary checking increment */
-	  if (octant == 2 || octant == 6)
-	    dfx = -dfx + ddf;
-	  else
-	    dfy = -dfy + ddf;
 	}
       octant++;
       if (octant > 8) octant -= 8;
       octantCount--;
     }
-
+  
 done:				/* jump out of two levels */
   
   for (i=0; i<b->height; i++)
@@ -739,15 +735,20 @@ blob_ellipse (double xc, double yc, double xp, double yp, double xq, double yq)
   /* Now offset the ellipse so that the center is exact, but the
      starting point is no longer exactly on the ellipse */
 
-  dpx = ROUND(xp+xc)-xp-xc;
+  /* This needs a change to blob_conic to work. blob_conic assumes
+   * we start at (0,0)
+   */
+
+  /*  dpx = ROUND(xp+xc)-xp-xc;
   dpy = ROUND(yp+yc)-yp-yc;
 
   F += dpx*(A*dpx+D+B*dpy) + dpy*(C*dpy+E);
   D += 2*A*dpx + B*dpy;
   E += B*dpx + 2*C*dpy;
-
+  */
   height = sqrt(A);
   y = floor(yc-height-0.5);
+
   /* We allow an extra pixel of slop on top and bottom to deal with
      round-off error */
   r = blob_new (y-1,ceil(yc+height+0.5)-y+3);
@@ -755,8 +756,20 @@ blob_ellipse (double xc, double yc, double xp, double yp, double xq, double yq)
   /* Although it seems that multiplying A-F by a constant would improve
      things, this seems not to be the case in practice. Several test
      cases showed about equal improvement and degradation */
-  blob_conic (r,ROUND(xp+xc),ROUND(yp+yc),ROUND(16*A),ROUND(16*B),ROUND(16*C),
-	      ROUND(16*D),ROUND(16*E),ROUND(16*F));
+  blob_conic (r,ROUND(xp+xc),ROUND(yp+yc),ROUND(128*A),ROUND(128*B),ROUND(128*C),
+	      ROUND(128*D),ROUND(128*E),ROUND(128*F));
+
+  /* Add a line through the center to improve things a bit. (Doesn't
+   * work perfectly because sometimes we overshoot
+   */
+  {
+    int x0 = floor (0.5 + xc - xp);
+    int x1 = floor (0.5 + xc + xp);
+    int y0 = floor (0.5 + yc - yp);
+    int y1 = floor (0.5 + yc + yp);
+ /*    r = blob_new (MIN(y0,y1), abs(y1-y0)+1); */
+    blob_line (r, x0, y0, x1, y1);
+  }
 
   return r;
 }
