@@ -294,6 +294,14 @@ area_motion_notify_event (GtkWidget      *widget,
 
   if (info.tool->use_threshhold)
     {
+      /* Work around bug in old XFree86 drivers, where a pen lifted
+       * from the tablet reports a pressure of 0.5
+       */
+      if ((event->pressure > 0.25) && !(event->state & GDK_BUTTON1_MASK))
+	{
+	  event->pressure = 0.0;
+	}
+
       if ((event->pressure >= info.tool->threshhold) && 
 	  !(info.tool->state.buttons & (1 << 1)))
 	{
